import * as VAR from '@eslint-react/var';
import { AST_NODE_TYPES } from '@typescript-eslint/types';
import { match, P } from 'ts-pattern';
import * as AST from '@eslint-react/ast';
import { constTrue } from '@eslint-react/eff';

// src/attribute/attribute.ts
function toString(node) {
  switch (node.type) {
    case AST_NODE_TYPES.JSXIdentifier:
      return node.name;
    case AST_NODE_TYPES.JSXNamespacedName:
      return `${node.namespace.name}:${node.name.name}`;
    case AST_NODE_TYPES.JSXMemberExpression:
      return `${toString(node.object)}.${toString(node.property)}`;
    case AST_NODE_TYPES.JSXText:
      return node.value;
    case AST_NODE_TYPES.JSXOpeningElement:
      return `<${toString(node.name)}>`;
    case AST_NODE_TYPES.JSXClosingElement:
      return `</${toString(node.name)}>`;
    case AST_NODE_TYPES.JSXOpeningFragment:
      return "<>";
    case AST_NODE_TYPES.JSXClosingFragment:
      return "</>";
  }
}

// src/attribute/attribute-name.ts
function getAttributeName(node) {
  return toString(node.name);
}

// src/attribute/attribute.ts
function getAttribute(name, attributes, initialScope) {
  return attributes.findLast((attr) => {
    if (attr.type === AST_NODE_TYPES.JSXAttribute) {
      return getAttributeName(attr) === name;
    }
    if (initialScope == null) return false;
    switch (attr.argument.type) {
      case AST_NODE_TYPES.Identifier: {
        const variable = VAR.findVariable(attr.argument.name, initialScope);
        const variableNode = VAR.getVariableInitNode(variable, 0);
        if (variableNode?.type === AST_NODE_TYPES.ObjectExpression) {
          return VAR.findPropertyInProperties(name, variableNode.properties, initialScope) != null;
        }
        return false;
      }
      case AST_NODE_TYPES.ObjectExpression:
        return VAR.findPropertyInProperties(name, attr.argument.properties, initialScope) != null;
    }
    return false;
  });
}
function getAttributeValue(node, name, initialScope) {
  switch (node.type) {
    case AST_NODE_TYPES.JSXAttribute:
      if (node.value?.type === AST_NODE_TYPES.Literal) {
        return {
          kind: "some",
          node: node.value,
          initialScope,
          value: node.value.value
        };
      }
      if (node.value?.type === AST_NODE_TYPES.JSXExpressionContainer) {
        return VAR.toStaticValue({
          kind: "lazy",
          node: node.value.expression,
          initialScope
        });
      }
      return { kind: "none", node, initialScope };
    case AST_NODE_TYPES.JSXSpreadAttribute: {
      const staticValue = VAR.toStaticValue({
        kind: "lazy",
        node: node.argument,
        initialScope
      });
      if (staticValue.kind === "none") {
        return staticValue;
      }
      return match(staticValue.value).with({ [name]: P.select(P.any) }, (value) => ({
        kind: "some",
        node: node.argument,
        initialScope,
        value
      })).otherwise(() => ({ kind: "none", node, initialScope }));
    }
    default:
      return { kind: "none", node, initialScope };
  }
}

// src/attribute/has.ts
function hasAttribute(name, attributes, initialScope) {
  return getAttribute(name, attributes, initialScope) != null;
}
function hasAnyAttribute(names, attributes, initialScope) {
  return names.some((n) => hasAttribute(n, attributes, initialScope));
}
function hasEveryAttribute(names, attributes, initialScope) {
  return names.every((n) => hasAttribute(n, attributes, initialScope));
}
function findParentAttribute(node, test = constTrue) {
  const guard = (node2) => {
    return node2.type === AST_NODE_TYPES.JSXAttribute && test(node2);
  };
  return AST.findParentNode(node, guard);
}
function getElementType(node) {
  if (node.type === AST_NODE_TYPES.JSXFragment) {
    return "";
  }
  return toString(node.openingElement.name);
}
function isHostElement(node) {
  return node.type === AST_NODE_TYPES.JSXElement && node.openingElement.name.type === AST_NODE_TYPES.JSXIdentifier && /^[a-z]/u.test(node.openingElement.name.name);
}
function isKeyedElement(node, initialScope) {
  return node.type === AST_NODE_TYPES.JSXElement && hasAttribute("key", node.openingElement.attributes, initialScope);
}
function isFragmentElement(node, allowJSXFragment = false) {
  if (node == null) return false;
  if (node.type !== AST_NODE_TYPES.JSXElement && node.type !== AST_NODE_TYPES.JSXFragment) return false;
  if (node.type === AST_NODE_TYPES.JSXFragment) return allowJSXFragment;
  return getElementType(node).split(".").at(-1) === "Fragment";
}

// src/jsx-detection-hint.ts
var JSXDetectionHint = {
  None: 0n,
  SkipUndefined: 1n << 0n,
  SkipNullLiteral: 1n << 1n,
  SkipBooleanLiteral: 1n << 2n,
  SkipStringLiteral: 1n << 3n,
  SkipNumberLiteral: 1n << 4n,
  SkipBigIntLiteral: 1n << 5n,
  SkipEmptyArray: 1n << 6n,
  SkipCreateElement: 1n << 7n,
  StrictArray: 1n << 8n,
  StrictLogical: 1n << 9n,
  StrictConditional: 1n << 10n
};
var DEFAULT_JSX_DETECTION_HINT = 0n | JSXDetectionHint.SkipUndefined | JSXDetectionHint.SkipBooleanLiteral;

// src/jsx-detection.ts
function isJsxText(node) {
  if (node == null) return false;
  return node.type === AST_NODE_TYPES.JSXText || node.type === AST_NODE_TYPES.Literal;
}
function isJsxLike(code, node, hint = DEFAULT_JSX_DETECTION_HINT) {
  switch (node?.type) {
    case AST_NODE_TYPES.JSXText:
    case AST_NODE_TYPES.JSXElement:
    case AST_NODE_TYPES.JSXFragment:
    case AST_NODE_TYPES.JSXAttribute:
    case AST_NODE_TYPES.JSXClosingElement:
    case AST_NODE_TYPES.JSXClosingFragment:
    case AST_NODE_TYPES.JSXEmptyExpression:
    case AST_NODE_TYPES.JSXExpressionContainer:
    case AST_NODE_TYPES.JSXIdentifier:
    case AST_NODE_TYPES.JSXMemberExpression:
    case AST_NODE_TYPES.JSXOpeningElement:
    case AST_NODE_TYPES.JSXOpeningFragment:
    case AST_NODE_TYPES.JSXSpreadAttribute:
    case AST_NODE_TYPES.JSXSpreadChild:
    case AST_NODE_TYPES.JSXNamespacedName: {
      return true;
    }
    case AST_NODE_TYPES.Literal: {
      switch (typeof node.value) {
        case "boolean":
          return !(hint & JSXDetectionHint.SkipBooleanLiteral);
        case "string":
          return !(hint & JSXDetectionHint.SkipStringLiteral);
        case "number":
          return !(hint & JSXDetectionHint.SkipNumberLiteral);
        case "bigint":
          return !(hint & JSXDetectionHint.SkipBigIntLiteral);
      }
      if (node.value == null) {
        return !(hint & JSXDetectionHint.SkipNullLiteral);
      }
      return false;
    }
    case AST_NODE_TYPES.TemplateLiteral: {
      return !(hint & JSXDetectionHint.SkipStringLiteral);
    }
    case AST_NODE_TYPES.ArrayExpression: {
      if (hint & JSXDetectionHint.StrictArray) {
        return node.elements.every((n) => isJsxLike(code, n, hint));
      }
      return node.elements.some((n) => isJsxLike(code, n, hint));
    }
    case AST_NODE_TYPES.LogicalExpression: {
      if (hint & JSXDetectionHint.StrictLogical) {
        return isJsxLike(code, node.left, hint) && isJsxLike(code, node.right, hint);
      }
      return isJsxLike(code, node.left, hint) || isJsxLike(code, node.right, hint);
    }
    case AST_NODE_TYPES.ConditionalExpression: {
      let leftHasJSX2 = function(node2) {
        if (Array.isArray(node2.consequent)) {
          if (node2.consequent.length === 0) {
            return !(hint & JSXDetectionHint.SkipEmptyArray);
          }
          if (hint & JSXDetectionHint.StrictArray) {
            return node2.consequent.every((n) => isJsxLike(code, n, hint));
          }
          return node2.consequent.some((n) => isJsxLike(code, n, hint));
        }
        return isJsxLike(code, node2.consequent, hint);
      }, rightHasJSX2 = function(node2) {
        return isJsxLike(code, node2.alternate, hint);
      };
      if (hint & JSXDetectionHint.StrictConditional) {
        return leftHasJSX2(node) && rightHasJSX2(node);
      }
      return leftHasJSX2(node) || rightHasJSX2(node);
    }
    case AST_NODE_TYPES.SequenceExpression: {
      const exp = node.expressions.at(-1);
      return isJsxLike(code, exp, hint);
    }
    case AST_NODE_TYPES.CallExpression: {
      if (hint & JSXDetectionHint.SkipCreateElement) {
        return false;
      }
      switch (node.callee.type) {
        case AST_NODE_TYPES.Identifier:
          return node.callee.name === "createElement";
        case AST_NODE_TYPES.MemberExpression:
          return node.callee.property.type === AST_NODE_TYPES.Identifier && node.callee.property.name === "createElement";
      }
      return false;
    }
    case AST_NODE_TYPES.Identifier: {
      const { name } = node;
      if (name === "undefined") {
        return !(hint & JSXDetectionHint.SkipUndefined);
      }
      if (AST.isJSXTagNameExpression(node)) {
        return true;
      }
      const variable = VAR.findVariable(name, code.getScope(node));
      const variableNode = variable && VAR.getVariableInitNode(variable, 0);
      return !!variableNode && isJsxLike(code, variableNode, hint);
    }
  }
  return false;
}

export { DEFAULT_JSX_DETECTION_HINT, JSXDetectionHint, findParentAttribute, getAttribute, getAttributeName, getAttributeValue, getElementType, hasAnyAttribute, hasAttribute, hasEveryAttribute, isFragmentElement, isHostElement, isJsxLike, isJsxText, isKeyedElement, toString };
